@@ -9,18 +9,18 @@
 """
 Editra Control Library: OutputBuffer
 
-This module contains classes that are usefull for displaying output from running
-tasks and processes. The classes are divided into three main catagories, gui
+This module contains classes that are useful for displaying output from running
+tasks and processes. The classes are divided into three main categories, gui
 classes, mixins, and thread classes. All the classes can be used together to
-easily create multithreaded gui display classes without neededing to worry about
+easily create multithreaded gui display classes without needing to worry about
 the details and thread safety of the gui.
 
 For example usage of these classes see ed_log and the Editra's Launch plugin
 
 Class OutputBuffer:
-This is the main class exported by ths module. It provides a readonly output
+This is the main class exported by this module. It provides a readonly output
 display buffer that when used with the other classes in this module provides an
-easy way to display continous output from other processes and threads. It
+easy way to display continuous output from other processes and threads. It
 provides two methods for subclasses to override if they wish to perform custom
 handling.
 
@@ -37,11 +37,11 @@ OutputBuffer is used with a L{ProcessThread}. It provides three methods that can
 be overridden in subclasses to perform extra processing.
 
   - DoProcessStart: Called as the process is being started in the ProcessThread,
-                    it recieves the process command string as an argument.
+                    it receives the process command string as an argument.
   - DoFilterInput: Called as each chunk of output comes from the running process
                    use it to filter the results before displaying them in the
                    buffer.
-  - DoProcessExit: Called when the running process has exited. It recieves the
+  - DoProcessExit: Called when the running process has exited. It receives the
                    processes exit code as a parameter.
 
 Class ProcessThread:
@@ -49,7 +49,7 @@ Thread class for running subprocesses and posting the output to an
 L{OutputBuffer} via events.
 
 Class TaskThread:
-Thread class for running a callable. For optimal performance and resposiveness
+Thread class for running a callable. For optimal performance and responsiveness
 the callable should be a generator object. All results are directed to an
 L{OutputBuffer} through its AppendUpdate method.
 
@@ -61,8 +61,19 @@ Requirements:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: outbuff.py 57103 2008-12-04 03:23:35Z CJP $"
-__revision__ = "$Revision: 57103 $"
+__svnid__ = "$Id: outbuff.py 63558 2010-02-26 01:13:41Z CJP $"
+__revision__ = "$Revision: 63558 $"
+
+__all__ = ["OutputBuffer", "OutputBufferEvent", "ProcessBufferMixin",
+           "ProcessThread", "TaskThread", "OPB_STYLE_DEFAULT", "OPB_STYLE_INFO",
+           "OPB_STYLE_WARN", "OPB_STYLE_ERROR", "OPB_STYLE_MAX",
+
+           "OPB_ERROR_NONE", "OPB_ERROR_INVALID_COMMAND",
+
+           "edEVT_PROCESS_START", "EVT_PROCESS_START", "edEVT_TASK_START",
+           "EVT_TASK_START", "edEVT_UPDATE_TEXT", "EVT_UPDATE_TEXT",
+           "edEVT_PROCESS_EXIT", "EVT_PROCESS_EXIT", "edEVT_TASK_COMPLETE",
+           "EVT_TASK_COMPLETE", "edEVT_PROCESS_ERROR", "EVT_PROCESS_ERROR"]
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -72,6 +83,7 @@ import time
 import errno
 import signal
 import threading
+import types
 import subprocess
 import wx
 import wx.stc
@@ -81,6 +93,7 @@ if subprocess.mswindows:
     import msvcrt
     import ctypes
 else:
+    import shlex
     import select
     import fcntl
 
@@ -101,9 +114,13 @@ OPB_ALL_STYLES = (wx.stc.STC_STYLE_DEFAULT, wx.stc.STC_STYLE_CONTROLCHAR,
                   OPB_STYLE_DEFAULT, OPB_STYLE_ERROR, OPB_STYLE_INFO,
                   OPB_STYLE_WARN)
 
+# Error Codes
+OPB_ERROR_NONE            = 0
+OPB_ERROR_INVALID_COMMAND = -1
+
 #--------------------------------------------------------------------------#
 
-# Event for notifying that the proces has started running
+# Event for notifying that the process has started running
 # GetValue will return the command line string that started the process
 edEVT_PROCESS_START = wx.NewEventType()
 EVT_PROCESS_START = wx.PyEventBinder(edEVT_PROCESS_START, 1)
@@ -126,12 +143,19 @@ EVT_PROCESS_EXIT = wx.PyEventBinder(edEVT_PROCESS_EXIT, 1)
 edEVT_TASK_COMPLETE = wx.NewEventType()
 EVT_TASK_COMPLETE = wx.PyEventBinder(edEVT_TASK_COMPLETE, 1)
 
+# Event to notify that an error occurred in the process
+edEVT_PROCESS_ERROR = wx.NewEventType()
+EVT_PROCESS_ERROR = wx.PyEventBinder(edEVT_PROCESS_ERROR, 1)
+
 class OutputBufferEvent(wx.PyCommandEvent):
     """Event for data transfer and signaling actions in the L{OutputBuffer}"""
     def __init__(self, etype, eid=wx.ID_ANY, value=''):
         """Creates the event object"""
         wx.PyCommandEvent.__init__(self, etype, eid)
+
+        # Attributes
         self._value = value
+        self._errmsg = None
 
     def GetValue(self):
         """Returns the value from the event.
@@ -140,6 +164,24 @@ class OutputBufferEvent(wx.PyCommandEvent):
         """
         return self._value
 
+    def GetErrorMessage(self):
+        """Get the error message value
+        @return: Exception traceback string or None
+
+        """
+        return self._errmsg
+
+    def SetErrorMessage(self, msg):
+        """Set the error message value
+        @param msg: Exception traceback string
+
+        """
+        try:
+            tmsg = unicode(msg)
+        except:
+            tmsg = None
+        self._errmsg = msg
+
 #--------------------------------------------------------------------------#
 
 class OutputBuffer(wx.stc.StyledTextCtrl):
@@ -406,8 +448,8 @@ class OutputBuffer(wx.stc.StyledTextCtrl):
     def RefreshBufferedLines(self):
         """Refresh and readjust the lines in the buffer to fit the current
         line buffering limits.
-        @poscondition: Oldest lines are removed until we are back within the
-                       buffer limit bounds.
+        @postcondition: Oldest lines are removed until we are back within the
+                        buffer limit bounds.
 
         """
         if self._line_buffer < 0:
@@ -539,6 +581,11 @@ class ProcessBufferMixin:
         self.Bind(EVT_PROCESS_START, self._OnProcessStart)
         self.Bind(EVT_UPDATE_TEXT, self._OnProcessUpdate)
         self.Bind(EVT_PROCESS_EXIT, self._OnProcessExit)
+        self.Bind(EVT_PROCESS_ERROR, self._OnProcessError)
+
+    def _OnProcessError(self, evt):
+        """Handle EVT_PROCESS_ERROR"""
+        self.DoProcessError(evt.GetValue(), evt.GetErrorMessage())
 
     def _OnProcessExit(self, evt):
         """Handles EVT_PROCESS_EXIT"""
@@ -564,11 +611,22 @@ class ProcessBufferMixin:
         """
         return txt
 
+    def DoProcessError(self, code, excdata=None):
+        """Override this method to do any ui notification of when errors happen
+        in running the process.
+        @param code: an OBP error code
+        @keyword excdata: Exception Data from process error
+        @return: None
+
+        """
+        pass
+
     def DoProcessExit(self, code=0):
         """Override this method to do any post processing after the running
         task has exited. Typically this is a good place to call
         L{OutputBuffer.Stop} to stop the buffers timer.
         @keyword code: Exit code of program
+        @return: None
 
         """
         self.Stop()
@@ -577,6 +635,7 @@ class ProcessBufferMixin:
         """Override this method to do any pre processing before starting
         a processes output.
         @keyword cmd: Command used to start program
+        @return: None
 
         """
         pass
@@ -599,14 +658,15 @@ class ProcessThread(threading.Thread):
 
     """
     def __init__(self, parent, command, fname='',
-                 args=list(), cwd=None, env=None):
+                 args=list(), cwd=None, env=dict(),
+                 use_shell=True):
         """Initialize the ProcessThread object
         Example:
           >>> myproc = ProcessThread(myframe, '/usr/local/bin/python',
                                      'hello.py', '--version', '/Users/me/home/')
           >>> myproc.start()
 
-        @param parent: Parent Window/EventHandler to recieve the events
+        @param parent: Parent Window/EventHandler to receive the events
                        generated by the process.
         @param command: Command string to execute as a subprocess.
         @keyword fname: Filename or path to file to run command on.
@@ -614,6 +674,8 @@ class ProcessThread(threading.Thread):
         @keyword cwd: Directory to execute process from or None to use current
         @keyword env: Environment to run the process in (dictionary) or None to
                       use default.
+        @keyword use_shell: Specify whether a shell should be used to launch 
+                            program or run directly
 
         """
         threading.Thread.__init__(self)
@@ -627,14 +689,23 @@ class ProcessThread(threading.Thread):
         self._parent = parent       # Parent Window/Event Handler
         self._cwd = cwd             # Path at which to run from
         self._cmd = dict(cmd=command, file=fname, args=args)
-        self._env = env
+        self._use_shell = use_shell
+        self._sig_abort = signal.SIGTERM    # default signal to kill process
+
+        # Make sure the environment is sane it must be all strings
+        nenv = dict(env) # make a copy to manipulate
+        for k, v in env.iteritems():
+            if isinstance(v, types.UnicodeType):
+                nenv[k] = v.encode(sys.getfilesystemencoding())
+            elif not isinstance(v, basestring):
+                nenv.pop(k)
+        self._env = nenv
 
         # Setup
         self.setDaemon(True)
 
     def __DoOneRead(self):
         """Read one line of output and post results.
-        @param proc: process to read from
         @return: bool (True if more), (False if not)
 
         """
@@ -719,7 +790,7 @@ class ProcessThread(threading.Thread):
 
             # Try to kill the group
             try:
-                os.kill(pid, signal.SIGKILL)
+                os.kill(pid, self._sig_abort)
             except OSError, msg:
                 pass
 
@@ -743,8 +814,9 @@ class ProcessThread(threading.Thread):
             ctypes.windll.kernel32.CloseHandle(handle)
 
     #---- Public Member Functions ----#
-    def Abort(self):
+    def Abort(self, sig=signal.SIGTERM):
         """Abort the running process and return control to the main thread"""
+        self._sig_abort = sig
         self.abort = True
 
     def run(self):
@@ -754,25 +826,46 @@ class ProcessThread(threading.Thread):
         @note: overridden from Thread
 
         """
+        # using shell, Popen will need a string, else it must be a sequence
+        # use shlex for complex command line tokenization/parsing
         command = u' '.join([item.strip() for item in [self._cmd['cmd'],
                                                        self._cmd['file'],
                                                        self._cmd['args']]])
+        command = command.strip()
+        if not self._use_shell and not subprocess.mswindows:
+            # shlex does not support unicode
+            command = shlex.split(command.encode(sys.getfilesystemencoding()))
 
-        use_shell = not subprocess.mswindows
-        self._proc = subprocess.Popen(command.strip(),
-                                      stdout=subprocess.PIPE,
-                                      stderr=subprocess.STDOUT,
-                                      shell=use_shell,
-                                      cwd=self._cwd,
-                                      env=self._env)
+        if sys.platform.lower().startswith('win'):            
+            suinfo = subprocess.STARTUPINFO()
+            suinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+        else:
+            suinfo = None
+        
+        err = None
+        try:
+            self._proc = subprocess.Popen(command,
+                                          stdout=subprocess.PIPE,
+                                          stderr=subprocess.STDOUT,
+                                          shell=self._use_shell,
+                                          cwd=self._cwd,
+                                          env=self._env,
+                                          startupinfo=suinfo)
+        except OSError, msg:
+            # NOTE: throws WindowsError on Windows which is a subclass of
+            #       OSError, so it will still get caught here.
+            err =  OutputBufferEvent(edEVT_PROCESS_ERROR,
+                                     self._parent.GetId(),
+                                     OPB_ERROR_INVALID_COMMAND)
+            err.SetErrorMessage(msg)
 
         evt = OutputBufferEvent(edEVT_PROCESS_START,
                                 self._parent.GetId(),
-                                command.strip())
+                                command)
         wx.PostEvent(self._parent, evt)
 
         # Read from stdout while there is output from process
-        while True:
+        while not err and True:
             if self.abort:
                 self.__KillPid(self._proc.pid)
                 self.__DoOneRead()
@@ -790,12 +883,17 @@ class ProcessThread(threading.Thread):
                 if not more:
                     break
 
-        try:
-            result = self._proc.wait()
-        except OSError:
+        # Notify of error in running the process
+        if err is not None:
+            wx.PostEvent(self._parent, err)
             result = -1
+        else:
+            try:
+                result = self._proc.wait()
+            except OSError:
+                result = -1
 
-        # Notify that proccess has exited
+        # Notify that process has exited
         # Pack the exit code as the events value
         evt = OutputBufferEvent(edEVT_PROCESS_EXIT, self._parent.GetId(), result)
         wx.PostEvent(self._parent, evt)
