@@ -21,8 +21,8 @@ LANGUAGE: Python
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: synextreg.py 63255 2010-01-25 02:05:22Z CJP $"
-__revision__ = "$Revision: 63255 $"
+__svnid__ = "$Id: synextreg.py 64581 2010-06-13 13:55:57Z CJP $"
+__revision__ = "$Revision: 64581 $"
 
 #-----------------------------------------------------------------------------#
 import os
@@ -374,7 +374,7 @@ EXT_MAP = {
            'bas cls ctl frm vb' : LANG_VB,
            'vbs dsm'            : LANG_VBSCRIPT,
            'v'                  : LANG_VERILOG,
-           'vh vhdl'            : LANG_VHDL,
+           'vh vhdl vhd'        : LANG_VHDL,
            'axl dtd plist rdf svg xml xrc xsd xsl xslt xul' : LANG_XML,
            'yaml yml'           : LANG_YAML,
            'groovy'             : LANG_GROOVY,
@@ -386,9 +386,9 @@ EXT_MAP = {
 
 class ExtensionRegister(dict):
     """A data storage class for managing mappings of
-    file types to file extentions. The register is created
+    file types to file extensions. The register is created
     as a singleton.
-    @status: initial implimentation
+    @status: initial implementation
 
     """
     instance = None
@@ -536,7 +536,7 @@ class ExtensionRegister(dict):
 
     def Remove(self, ftype):
         """Remove a filetype from the register
-        @param ftype: File tyep description string
+        @param ftype: File type description string
         @return: bool removed or not
 
         """
